#include <avr/io.h>
#include <string.h>
#include <stdio.h>
//#include <util/delay.h>
#include <avr/interrupt.h>
//#include <util/twi.h>
#include <inttypes.h>

#include "isdn-display.h"
#include "ds18x20.h"
#include "onewire.h"
#include "crc8.h"
#include "delay.h"
#include "i2cmaster.h"
#include "rtc.h"


#define EEPROM_ADDR	0xA0   // device address of EEPROM , see datasheet

#define UHRSTELLEN 0
#define SETDATE "04.06.06"
#define SETTIME "15:07:00"


#define MAXSENSORS 10
uint8_t gSensorIDs[MAXSENSORS][OW_ROMCODE_SIZE];
char buffer[40+1];

// ************** search_sensors ***************
uint8_t search_sensors(void)
{
	uint8_t i;
	uint8_t id[OW_ROMCODE_SIZE];
	uint8_t diff, nSensors;
	
	
	//lcd_gotoxy(0,3);lcd_puts("Scanning Bus for DS18X20");
	
	nSensors = 0;
	
	for( diff = OW_SEARCH_FIRST; 
		diff != OW_LAST_DEVICE && nSensors < MAXSENSORS ; )
	{
		DS18X20_find_sensor( &diff, &id[0] );
		
		if( diff == OW_PRESENCE_ERR ) {
			lcd_gotoxy(3,3);lcd_puts("No Sensor found" );
			break;
		}
		
		if( diff == OW_DATA_ERR ) {
			lcd_gotoxy(3,3);lcd_puts( "Bus Error" );
			break;
		}
		
		for (i=0;i<OW_ROMCODE_SIZE;i++)
			gSensorIDs[nSensors][i]=id[i];

		//sprintf(buffer,"RC: %u", id[0]);
		//lcd_gotoxy(0,3);lcd_puts(buffer);
		
		nSensors++;
	}
	
	return nSensors;
}	

// ************** ReadChannel ***************
uint16_t ReadChannel(uint8_t mux)  //zero-based
{
  uint8_t i;
  uint16_t result = 0;         //Initialisieren wichtig, da lokale Variablen
                               //nicht automatisch initialisiert werden und
                               //zufllige Werte haben. Sonst kann Quatsch rauskommen
  ADCSRA = (1<<ADEN) | (1<<ADPS2) |(1<<ADPS1) | (1<<ADPS0);    // Frequenzvorteiler 
                               // setzen auf 8 (1) und ADC aktivieren (1)

  ADMUX=0x0;
  ADMUX |= mux;                      // Kanal waehlen
  ADMUX &= ~(1<<REFS1);
  ADMUX &= ~(1<<REFS0); // externe Referenzspannung nutzen 

	//  sprintf(buffer,"ADMUX: %d ",ADMUX);
	//lcd_gotoxy(20,0); lcd_puts(buffer);
	//delay_ms(200);

  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
  ADCSRA |= (1<<ADSC);              // eine ADC-Wandlung 
  while ( ADCSRA & (1<<ADSC) );     // auf Abschluss der Konvertierung warten 

  /* Eigentliche Messung - Mittelwert aus 4 aufeinanderfolgenden Wandlungen */
  for(i=0;i<4;i++)
  {
    ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
    while ( ADCSRA & (1<<ADSC) );   // auf Abschluss der Konvertierung warten
    result += ADCW;		    // Wandlungsergebnisse aufaddieren
	
	
  }
  ADCSRA &= ~(1<<ADEN);             // ADC deaktivieren (2)

  result /= 4;                     // Summe durch vier teilen = arithm. Mittelwert

  //sprintf(buffer,"RC: %u ",result);
  //lcd_gotoxy(15,0); lcd_puts(buffer);


  return result;
}

//************ eeprom_writeData ********
void eeprom_writeData(uint8_t cel, uint8_t celfrac, uint8_t code)
{
}

// ************ main ***********
int main(void)
{

	
	uint8_t nSensors, i;
	uint8_t subzero, cel, cel_frac_bits;
	uint8_t Stunden, Minuten, Sekunden;
	uint8_t Tag, Monat, Jahr;
	
	uint8_t ret;
	

	lcd_init(LCD_DISP_ON);
	i2c_init();                             // initialize I2C library 
	
 
	
	
	
	#ifndef OW_ONE_BUS
	//ow_set_bus(&PINB,&PORTB,&DDRB,PB0);
	#endif
	
	sei();
	

	// Dann den ersten Sensor suchen
	ow_reset();

	nSensors = search_sensors();

	
	if (UHRSTELLEN)
	{
	  rtc_init();  // nur bei DS1307, nicht bei DS1337
	  rtc_initClock(SETDATE, SETTIME);

	} 

	// Hauptschleife
	while(1)
	{	

		if (DS18X20_OK!=DS18X20_start_meas( DS18X20_POWER_EXTERN, NULL ))
		{  		lcd_gotoxy(37,0); 
				lcd_puts("E01");
		}

		delay_ms(DS18B20_TCONV_12BIT);

		ReadDate(&Jahr, &Monat, &Tag);

		ReadTime(&Stunden, &Minuten, &Sekunden);

		for(i=0;i<nSensors;i++)
		{  

			if (DS18X20_OK!=DS18X20_read_meas(gSensorIDs[i], &subzero, &cel, &cel_frac_bits))
			{  		lcd_gotoxy(37,0);
	    			lcd_puts("E02");

			}
			
			if (subzero)
			{	sprintf(buffer,"T%i:-%.2i,%.2i %.2X",i+1,cel, cel_frac_bits, gSensorIDs[i][7]);
			    eeprom_writeData(cel+128,cel_frac_bits, gSensorIDs[i][7]);  // bei Minus einfach +128
			}
			else
			{	sprintf(buffer,"T%i: %.2i,%.2i %.2X",i+1,cel, cel_frac_bits, gSensorIDs[i][7]);
			    eeprom_writeData(cel,cel_frac_bits, gSensorIDs[i][7]);
			}

			if (i<2)
				lcd_gotoxy(i*15,1);
			else
			    lcd_gotoxy((i-2)*15,2);
	    	lcd_puts(buffer);
			
/*		int j;
		//if (i==0)
		{   sprintf(buffer,"c: ");	    
			for(j=0;j<OW_ROMCODE_SIZE;j++)
			  sprintf(buffer,"%s %X",buffer, gSensorIDs[i][j]);
			lcd_gotoxy(14,i+2);
	    	lcd_puts(buffer);
		}
*/
		} //for

/*		if (nSensors==1)
		{	i = gSensorIDs[0][0]; // family-code for conversion-routine

			sprintf(buffer,"RC: %i",gSensorIDs[0][0]);
			lcd_gotoxy(19,2);
	    	lcd_puts(buffer);
		
			if (DS18X20_OK!=DS18X20_start_meas( DS18X20_get_power_status(i), NULL ))
			{  		lcd_gotoxy(0,3);
	    			lcd_puts("Error start_meas");

			}

			delay_ms(DS18B20_TCONV_12BIT+100);
		
			if (DS18X20_OK!=DS18X20_read_meas_single(i, &subzero, &cel, &cel_frac_bits))
			{  		lcd_gotoxy(0,3);
	    			lcd_puts("Error read_meas_single");

			}
    

			if (subzero)
				sprintf(buffer,"Temp.: -%i,%i",cel, cel_frac_bits);
			else
				sprintf(buffer,"Temp.:  %i,%i", cel, cel_frac_bits);

			lcd_gotoxy(0,2);
	    	lcd_puts(buffer);
		}		
*/

	
     //TempWerte ins EEProm schreiben

     // write 0x75 to EEPROM address 5 (Byte Write) 
     i2c_start_wait(EEPROM_ADDR+I2C_WRITE);     // set device address and write mode
     i2c_write(0x00);
	 i2c_write(0x05);                        // write address = 5
     i2c_write(0x75);                        // write value 0x75 to EEPROM
     i2c_stop();                             // set stop conditon = release bus

     // read previously written value back from EEPROM address 5 
     i2c_start_wait(EEPROM_ADDR+I2C_WRITE);     // set device address and write mode
	 i2c_write(0x00);
     i2c_write(0x05);                        // write address = 5
     i2c_rep_start(EEPROM_ADDR+I2C_READ);       // set device address and read mode

     ret = i2c_readNak();                    // read one byte from EEPROM
     i2c_stop();


   	 

//	ReadDate(&Jahr, &Monat, &Tag);

//	ReadTime(&Stunden, &Minuten, &Sekunden);
 
	sprintf(buffer,"%.2i:%.2i:%.2i  %.2i.%.2i.20%.2i  ",Stunden, Minuten, Sekunden, Tag, Monat, Jahr);
	lcd_gotoxy(0,0); lcd_puts(buffer);


	// Start AD/Wandler
	uint16_t ad_out;
	double ad_erg;

	// Auslesen des Drucksensors von Freescale
	ad_out=ReadChannel(4); // PA4
	//sprintf(buffer,"AD1: %d ",ad_out);
	//lcd_gotoxy(15,1); lcd_puts(buffer);

	ad_erg = ad_out*5.0/1024.0; //AD-Wandlung

	//sprintf(buffer,"AD2: %.0f ",ad_erg);
	//lcd_gotoxy(15,2); lcd_puts(buffer);

	
    ad_erg=((ad_erg/5.0)+0.095)/0.009*10; // siehe Datenblatt 5100
	
	sprintf(buffer,"Luftdruck: %.0f hPa",ad_erg);
	lcd_gotoxy(0,3); lcd_puts(buffer);

	// TMP235 von National auslesen
/*	ad_out = ReadChannel(0); // PA0
	ad_erg = ad_out*5.0/1024.0; //AD-Wandlung

	sprintf(buffer,"TempX: %.2f ",ad_erg*10);  // TMP235 von National
	lcd_gotoxy(25,3); lcd_puts(buffer);
*/
		
	
/*	// Auslesen von Temp37
	ad_out = ReadChannel(0x0);
	//sprintf(buffer,"AD1: %d ",ad_out);
	//lcd_gotoxy(15,1); lcd_puts(buffer);

	ad_erg = ad_out*5.0/1024.0; //AD-Wandlung

    sprintf(buffer,"TMP37: %.2f ",ad_erg);
	lcd_gotoxy(0,0); lcd_puts(buffer);
*/
		delay_ms(1000);

	
	} //while
	

	
	
	
	//return 0;
}
